package dk.nsp.idsas.security.impl;

import java.util.LinkedList;
import java.util.List;
import java.util.function.Supplier;

import dk.sds.nsp.security.Security;
import dk.sds.nsp.security.SecurityContext;

import dk.nsp.idsas.security.IdsasActorProvider;
import dk.nsp.idsas.security.IdsasSecurityException;
import dk.nsp.idsas.security.model.IdsasActor;

public class IdsasActorProviderImpl implements IdsasActorProvider {
    // Formålet med at bruge en Supplier er, at vi kan udsætte "getSecurityContext"
    // til vi er i selve ServletRequest-tråden.
    final private Supplier<SecurityContext> securitySupplier;

    public IdsasActorProviderImpl() {
        this.securitySupplier = Security::getSecurityContext;
    }

    public IdsasActorProviderImpl(Supplier<SecurityContext> securitySupplier) {
        this.securitySupplier = securitySupplier;
    }

    @Override
    public IdsasActor getActor() throws IdsasSecurityException {
        SecurityContext securityContext = securitySupplier.get();

        // Der skal være en sikkerhedsbillet
        if (!securityContext.getTicket().isPresent()) {
            throw new IdsasSecurityException("Sikkerhedsbilletten mangler");
        }

        // Sikkerhedsbilletten skal være valid
        if (!securityContext.getTicket().get().isValid()) {
            throw new IdsasSecurityException("Sikkerhedsbilletten var ikke valid");
        }

        // Gennemløber de enkelte brugertyper
        List<IdsasActor> matchedActors = new LinkedList<>();
        IdsasActor system = matchSystemActor(securityContext);
        if (system != null) {
            matchedActors.add(system);
        }
        IdsasActor sundhedsfaglig = matchSundhedsfagligActor(securityContext);
        if (sundhedsfaglig != null) {
            matchedActors.add(sundhedsfaglig);
        }

        if (matchedActors.size() == 0) {
            throw new IdsasSecurityException("Brugertypen kunne ikke bestemmes ud fra den medsendte sikkerhedsbillet");
        } else if (matchedActors.size() > 1) {
            throw new IdsasSecurityException("Brugertypen kunne ikke entydigt bestemmes ud fra den medsendte sikkerhedsbillet");
        }
        return matchedActors.get(0);
    }

    private IdsasActor matchSystemActor(SecurityContext securityContext) {
        // Der skal ikke være en acting user, når det er en systembruger
        if (securityContext.getActingUser().isPresent()) return null;

        // Der skal ikke være en principal user, når det er en systembruger
        if (securityContext.getPrincipalUser().isPresent()) return null;

        // Systembruger skal have en organisation
        if (!securityContext.getOrganisation().isPresent()) return null;

        // Systembruger skal have en organisation af typen CVR
        if (!securityContext.getOrganisation().get().getIdentifierFormat().equals(
                SecurityContext.Organisation.OrganisationIdentifierFormat.CVR)) {
            return null;
        }

        // Systembruger skal have en organisation med en identifier
        String organisationIdentifier = securityContext.getOrganisation().get().getIdentifier();
        if (organisationIdentifier.isEmpty()) {
            return null;
        }

        return IdsasActor.createSystem(organisationIdentifier);
    }

    private IdsasActor matchSundhedsfagligActor(SecurityContext securityContext) {

        // Der skal ikke være en principal user, når det er en sundhedsfaglig
        if (securityContext.getPrincipalUser().isPresent()) return null;

        // Der skal være en ActingUser, når det er en sundhedsfaglig
        if (!securityContext.getActingUser().isPresent()) return null;

        // ActingUser skal have typen HealthcareProfessional
        if (!securityContext.getActingUser().get().getUserType().equals(
                SecurityContext.User.UserType.HealthcareProfessional)) return null;

        // ActingUser skal have en identifier
        if (securityContext.getActingUser().get().getIdentifier().isEmpty()) return null;

        // Identifier skal have format CPR
        if (!securityContext.getActingUser().get().getIdentifierFormat().equals(
                SecurityContext.User.PersonIdentifierFormat.CPR)) return null;

        // Sundhedsfaglig skal have en organisation
        if (!securityContext.getOrganisation().isPresent()) return null;

        // Sundhedsfaglig skal have en organisation af typen CVR
        if (!securityContext.getOrganisation().get().getIdentifierFormat().equals(
                SecurityContext.Organisation.OrganisationIdentifierFormat.CVR)) {
            return null;
        }

        // Sundhedsfaglig skal have en organisation med en identifier
        String organisationIdentifier = securityContext.getOrganisation().get().getIdentifier();
        if (organisationIdentifier.isEmpty()) {
            return null;
        }

        return IdsasActor.createSundhedsFaglig(organisationIdentifier);
    }
}
